/* Virginia Tech Cognitive Radio Open Source Systems
 * Virginia Tech, 2009
 *
 * LICENSE INFORMATION GOES HERE
 */

/* Implementation file for the VCROSS Cognitive Radio public API defined in
 * include/libvtcross.h.
 *
 * MORE INFO HERE
 */

#include <cstdlib>

#include "vtcross/common.h"
#include "vtcross/debug.h"
#include "vtcross/libvtcross.h"


uint32_t
ConnectToRemoteComponent()
{
   	uint32_t socket; 
    socket = ClientSocket("localhost", "40000");
    
    return socket;
}

uint32_t
GetNumObservables()
{
	uint32_t socketfd, numObservables;
    char buffer[256];

    socketfd = ConnectToRemoteComponent();
    SendMessage(socketfd,"get_number_observables");
    
    memset(buffer, 0, 256);
    ReadMessage(socketfd, buffer);
    numObservables = atoi(buffer);
	
	return numObservables; 
}

uint32_t
GetNumUtilities()
{
	uint32_t socketfd, numUtilities;
    char buffer[256];

    socketfd = ConnectToRemoteComponent();
    SendMessage(socketfd,"get_number_utilities");
    
    memset(buffer, 0, 256);
    ReadMessage(socketfd, buffer);
    numUtilities = atoi(buffer);
	
	return numUtilities; 
}

uint32_t
GetNumParameters()
{
	uint32_t socketfd, numParameters;
    char buffer[256];

    socketfd = ConnectToRemoteComponent();
    SendMessage(socketfd,"get_number_parameters");
    
    memset(buffer, 0, 256);
    ReadMessage(socketfd, buffer);
    numParameters = atoi(buffer);
	
	return numParameters; 
}

/* Given a certain set of observables, ask the radio to find the optimum radio
 * parameters and return them. 
 *
 * TODO I'm a little confused about this function... why would anyone need to
 * use this?  Shouldn't this be internal to the radio operation? 
 *
 * TODO this function is returning a pointer to allocated memory, which is fine,
 * but we need to document this and make sure the caller is deallocating the
 * memory when it is done using it.
 */
Parameter* GetOptimalParameters(Observable *radioObservables, uint32_t numObservables,
		Parameter *currentParameters, uint32_t numCurrentParameters) {
    uint32_t i,socketfd,numParameters;
    char var[50];
    char counter[55];
	char buffer[256];

    uint32_t socketFD = ConnectToRemoteComponent();
    SendMessage(socketFD, "request_optimization");

    /* Get number of observables to send.  This information needs to be 
     * sent to the Cognitive Radio Shell also.   
     */

    // Send Observables
    sprintf(counter, "%i", numObservables);
    SendMessage(socketFD, counter);
    for(size_t i = 0; i < numObservables; i++) {
        SendMessage(socketFD, radioObservables[i].name.c_str());
        sprintf(var, "%f", radioObservables[i].value);
        SendMessage(socketFD, var);	
    }

    // Send Parameters 
    memset(counter, 0, 55);
    sprintf(counter, "%i", numCurrentParameters);
    SendMessage(socketFD, counter);
	
	for(i = 0; i < numCurrentParameters; i++) {
        SendMessage(socketFD,currentParameters[i].name.c_str());
        sprintf(var,"%f",currentParameters[i].value);
        SendMessage(socketFD,var);	
    }

    /* Receive Set of Parameters */
    memset(buffer, 0, 256);
    ReadMessage(socketFD, buffer);
    numParameters = atoi(buffer);
    Parameter *pList = new Parameter[numParameters];
    
    for(ssize_t i = 0; i < numParameters; i++) {
        memset(buffer, 0, 256);
        ReadMessage(socketFD, buffer);
        pList[i].name = std::string(buffer);
        
		memset(buffer, 0, 256);
        ReadMessage(socketFD, buffer);
        pList[i].value = atof(buffer);
    }   

    return pList;
}

bool
UpdateParameterPerformance(Parameter *p, uint32_t numParameters, Observable *o,
		uint32_t numObservables)
{
	char counter[55];
    char var[50];

    uint32_t socketFD = ConnectToRemoteComponent();
    SendMessage(socketFD, "update_performance");
    
    // Send Parameters 
    memset(counter, 0, 55);
    sprintf(counter, "%i", numParameters);
    SendMessage(socketFD, counter);
	
	for(size_t i = 0; i < numParameters; i++) {
        SendMessage(socketFD,p[i].name.c_str());
        sprintf(var,"%f",p[i].value);
        SendMessage(socketFD,var);	
    }
    
	// Send Observables
    sprintf(counter, "%i", numObservables);
    SendMessage(socketFD, counter);
    for(size_t i = 0; i < numObservables; i++) {
        SendMessage(socketFD, o[i].name.c_str());
        sprintf(var, "%f", o[i].value);
        SendMessage(socketFD, var);	
    }
}
